//	Библиотека iarduino_I2C_connect разработана для удобства соединения нескольких arduino по шине I2C
//
//	Подключение библиотеки:
//	#include <Wire.h>
//	#include <iarduino_I2C_connect.h>
//	
//	Создание объекта библиотеки
//	iarduino_I2C_connect I2C2;               Вы можете указать своё название, вместо I2C2
//	
//	В библиотеке реализованы 4 функции:
//	
//	ДЛЯ ВЕДОМОГО:
//	
//	Функция begin     Назначение:            указание массива, элементы которого будут доступны для чтения/записи по шине I2C
//	                  Синтаксис:             begin(массив);
//	                  Параметр:              массив (тип byte)
//	                  Возвращаемые значения: Нет
//	                  Примечание:            Вызывается 1 раз в начале кода.
//	
//	Функция writeMask Назначение:            указание маскировочного массива, значение true элемента данного массива - разрешает менять значение соответствующего элемента в массиве, объявленном в функции begin()
//	                  Синтаксис:             writeMask(маскировочный_массив);
//	                  Параметр:              маскировочный массив (тип bool)
//	                  Возвращаемые значения: Нет
//	                  Примечание:            Вызывается 1 раз в начале кода.
//	                                         Если указать массив, который состоит из 5 ячеек {0,1,1,1,0}, то...
//	                                         Мастер не сможет изменить значения 0 и 4 элементов массива доступного по шине I2C, который был объявлен в функции begin(). Но все элементы массива будут доступны для чтения
//	                                         Если указать массив, который состоит из 5 ячеек, в то время как массив доступный по шине I2C содержит 10 ячеек, то...
//	                                         Мастер не сможет изменить значения последних 5 ячеек, а значения первых сможет изменить только если соответствующие ячейки маскировочного массива равны 1 или true
//	                                         Если не объявлять маскировочный массив (не вызывать функцию writeMask ), то...
//	                                         Все элементы массива, доступного по шине I2C, который был объявлен в функции begin(), будут доступны мастеру для записи и чтения.
//	
//	ДЛЯ ВЕДУЩЕГО:
//	
//	Функция readByte  Назначение:            Чтение одного байта данных из устройства на шине I2C
//	                  Синтаксис:             readByte(адрес_устройства, адрес_регистра);
//	                  Параметры:             адрес_устройства - данные которого требуется получить (число от 0x00 до 0x7F)
//	                                         адрес_регистра   - данные которого требуется получить (число от 0x00 до 0xFF) // адрес регистра соответствует номеру элемента массива объявленного ведомым устройством функцией begin
//	                  Возвращаемые значения: uint8_t байт     - данные считанные из указанного адреса, указанного устройства
//
//	Функция writeByte Назначение:            Запись одного байта данных в устройство на шине I2C
//	                  Синтаксис:             writeByte(адрес_устройства, адрес_регистра, байт_данных);
//	                  Параметры:             адрес_устройства - которому предназначены отправляемые данные (число от 0x00 до 0x7F)
//	                                         адрес_регистра   - которому предназначены отправляемые данные (число от 0x00 до 0xFF) // адрес регистра соответствует номеру элемента массива объявленного ведомым устройством функцией begin
//	                                         байт_данных      - который требуется записать в указанный регистр, указанного устройства
//	                  Возвращаемые значения: uint8_t байт     - результат операции записи, соответствует значению возвращаемому функцией Wire.endTransmission()
//	                                         0                - успех
//	                                         1                - переполнение буфера передачи данных
//	                                         2                - получен NACK на переданный адрес устройства
//	                                         3                - получен NACK на переданный адрес регистра
//	                                         4                - неизвестная ошибка

#ifndef iarduino_I2C_connect_h
#define iarduino_I2C_connect_h
#include <Wire.h>

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
								
class iarduino_I2C_connect{
	public:								uint8_t	readByte		(uint8_t, uint8_t);				//	чтение одного байта данных из устройства с указанием: (адрес устройства, адрес регистра)
	/**	пользовательские функции **/	uint8_t	writeByte		(uint8_t, uint8_t, uint8_t);	//	запись одного байта данных в  устройство с указанием: (адрес устройства, адрес регистра, байт для записи)
		template< typename T, size_t j>	void	begin			(T(&i)[j]){I2C_func_BEGIN(i,j);}//	инициализация в качестве slave, с указанием массива, данные кторого будут доступны для чтения/записи по шине I2C
		template< typename T, size_t j>	void	writeMask		(T(&i)[j]){I2C_func_MASK (i,j);}//	указание маскировочного массива, значение true элемента данного массива - разрешает менять значение соответствующего элемента в массиве I2C_array_REG по шине I2C
	private:							void	I2C_func_BEGIN	(uint8_t*, uint8_t);			//  внутренняя функция
	/**	внутренние функции **/			void	I2C_func_MASK	(bool*,    uint8_t);			//  внутренняя функция
};

class iarduino_I2C_connect_volatile_class{
	public:								uint8_t *I2C_array_REG	= 0;							//	указатель на массив, данные которого будут доступны для чтения/записи по шине I2C
	/**	указатели на массивы **/		uint8_t I2C_length_REG	= 0;							//	длина массива, данные которого будут доступны для чтения/записи по шине I2C
	/**	и значения их параметров **/	uint8_t I2C_index_REG	= 0;							//	индекс активного элемента массива (с которого начнётся чтение/запись)
										bool	*I2C_array_MASK	= 0;							//	указатель на маскировочный массив, значение true элемента данного массива - разрешает менять значение соответствующего элемента в массиве I2C_array_REG по шине I2C
										bool	I2C_length_MASK	= 0;							//	длина маскировочного массива
										bool	I2C_flag_MASK	= 0;							//	флаг, указывающий на то, что маскировочный массив был задан
};

#endif